`timescale 1ns/100ps

module axi_register_slice
#(
    parameter DATA_WIDTH = 32
) 
(
    input                   clk,
    input                   reset,
    input                   clk_en,
  
    input                   s_axi_valid,
    output                  s_axi_ready,
    input  [DATA_WIDTH-1:0] s_axi_data,
    
    output                  m_axi_valid,
    input                   m_axi_ready,
    output [DATA_WIDTH-1:0] m_axi_data
);
  ////////////////////////////////////////////////////////////////////
      localparam SM_RESET = 2'b00;
      localparam SM_IDLE  = 2'b10;
      localparam SM_ONE   = 2'b11;
      localparam SM_FULL  = 2'b01;
   
      reg [1:0] state = SM_RESET;
      reg [DATA_WIDTH-1:0] payload_a;
      reg [DATA_WIDTH-1:0] payload_b;
      reg sel_rd = 1'b0;
      reg sel_wr = 1'b0;
      wire sel;
      wire load_a;
      wire load_b;
      
      assign m_axi_valid = state[0];
      assign s_axi_ready = state[1];

      always @(posedge clk) begin 
        if (reset) begin
          state <= SM_RESET;
        end
        else if (clk_en) begin
          case (state)
            SM_IDLE:
              if (s_axi_valid)
                state <= SM_ONE;
            SM_ONE:
              if (s_axi_valid & ~m_axi_ready)
                state <= SM_FULL;
              else if (~s_axi_valid & m_axi_ready)
                state <= SM_IDLE;
            SM_FULL:
              if (m_axi_ready)
                state <= SM_ONE;
            // SM_RESET:
            default:
              state <= SM_IDLE;
          endcase
        end
      end

      assign sel = sel_rd; 

      always @(posedge clk) begin 
        if (reset) begin
          sel_rd <= 1'b0;
        end 
        else if (clk_en) begin
          sel_rd <= (m_axi_ready & m_axi_valid) ? ~sel_rd : sel_rd;
        end
      end

      assign load_a = ~sel_wr & (state != SM_FULL);
      assign load_b = sel_wr & (state != SM_FULL);
      always @(posedge clk) begin 
        if (reset) begin
          sel_wr <= 1'b0;
        end 
        else if (clk_en) begin
          sel_wr <= (s_axi_ready & s_axi_valid) ? ~sel_wr : sel_wr;
        end
      end

      always @(posedge clk) begin 
        if (clk_en) begin 
          payload_a <= load_a ? s_axi_data : payload_a;
        end
      end

      always @(posedge clk) begin 
        if (clk_en) begin 
          payload_b <= load_b ? s_axi_data : payload_b;
        end
      end

      assign m_axi_data = sel ? payload_b : payload_a;


  
endmodule