
`timescale 1 ns/1 ps

module axi_to_ll_reg #
(
   parameter            DATA_WIDTH         = 16, // DATA bus width
   parameter            STRB_WIDTH         = 2, // STROBE bus width
   parameter            BC                 = DATA_WIDTH>>3, //Byte count
   parameter            ISUFC              = 0, //If UFC send 1 
   parameter            REM_WIDTH          = 1 // REM bus width
)
( 
   // AXI4-S input signals
   AXI4_S_IP_TX_TVALID,
   AXI4_S_IP_TX_TREADY,
   AXI4_S_IP_TX_TDATA,
   AXI4_S_IP_TX_TKEEP,
   AXI4_S_IP_TX_TLAST,

   // LocalLink output Interface
   LL_OP_DATA,
   LL_OP_SOF_N,
   LL_OP_EOF_N,
   LL_OP_REM,
   LL_OP_SRC_RDY_N,
   LL_IP_DST_RDY_N,

   // System Interface
   USER_CLK,
   RESET,    
   CHANNEL_UP

);

 //***********************************Port Declarations******************************* 
    // AXI4-Stream Interface
    input   [(DATA_WIDTH-1):0]     AXI4_S_IP_TX_TDATA;
    input   [(STRB_WIDTH-1):0]     AXI4_S_IP_TX_TKEEP;
    input                          AXI4_S_IP_TX_TVALID;
    input                          AXI4_S_IP_TX_TLAST;
    output                         AXI4_S_IP_TX_TREADY;
 
    // LocalLink TX Interface
    output    [0:(DATA_WIDTH-1)]   LL_OP_DATA;
    output    [0:(REM_WIDTH-1)]    LL_OP_REM;
    output                         LL_OP_SRC_RDY_N;
    output                         LL_OP_SOF_N;
    output                         LL_OP_EOF_N;
    input                          LL_IP_DST_RDY_N;
 
    // System Interface
    input                          USER_CLK;
    input                          CHANNEL_UP;
    input                          RESET;
  
 //***********************************Signal Declarations*******************************  
    reg                            new_pkt_r; 
    wire                           new_pkt;

    wire                           LL_OP_SRC_RDY;
    wire                           LL_OP_SRC_RDY_INT;
    wire    [0:(DATA_WIDTH-1)]     LL_OP_DATA_INT;
    wire                           LL_OP_EOF;
    wire                           LL_OP_EOF_INT;
    reg     [0:(REM_WIDTH-1)]      LL_OP_REM_INT;
    wire    [0:(REM_WIDTH+1)]      LL_TUSER;
    wire    [0:(REM_WIDTH+1)]      LL_TUSER_INT;
    wire    [0:(REM_WIDTH+1)]      LL_TUSER_INT_i;
    wire   [0:(STRB_WIDTH-1)]      AXI4_S_IP_TX_TKEEP_i;
         
    wire   [(DATA_WIDTH-1):0]      AXI4_S_IP_TX_TDATA_m;
    wire   [(STRB_WIDTH-1):0]      AXI4_S_IP_TX_TKEEP_m;                            
    wire                           AXI4_S_IP_TX_TVALID_m;
    wire                           AXI4_S_IP_TX_TLAST_m;
    wire                           AXI4_S_IP_TX_TREADY_m;
    reg                            found_rem;
    reg [(STRB_WIDTH-1):0]         i_rem;

    wire                             s_axi_valid_reg;
    wire                             s_axi_ready_reg;
    wire [DATA_WIDTH+STRB_WIDTH:0]   s_axi_data_reg; 
    
    wire                             m_axi_valid_reg;
    wire                             m_axi_ready_reg;
    wire [DATA_WIDTH+STRB_WIDTH:0]   m_axi_data_reg; 

    wire                             s_axi_valid_reg0;
    wire                             s_axi_ready_reg0;
    wire [DATA_WIDTH+REM_WIDTH+1:0]  s_axi_data_reg0; 
    
    wire                             m_axi_valid_reg0;
    wire                             m_axi_ready_reg0;
    wire [DATA_WIDTH+REM_WIDTH+1:0]  m_axi_data_reg0; 
    
    wire                             LL_IP_DST_RDY;
    
//*********************************Main Body of Code**********************************
assign s_axi_valid_reg           = AXI4_S_IP_TX_TVALID;
assign AXI4_S_IP_TX_TREADY       = s_axi_ready_reg;
assign s_axi_data_reg            = {AXI4_S_IP_TX_TKEEP,AXI4_S_IP_TX_TLAST,AXI4_S_IP_TX_TDATA};
assign m_axi_ready_reg           = AXI4_S_IP_TX_TREADY_m;
assign AXI4_S_IP_TX_TVALID_m     = m_axi_valid_reg;
assign AXI4_S_IP_TX_TDATA_m      = m_axi_data_reg[DATA_WIDTH-1:00];
assign AXI4_S_IP_TX_TLAST_m      = m_axi_data_reg[DATA_WIDTH];
assign AXI4_S_IP_TX_TKEEP_m      = m_axi_data_reg[DATA_WIDTH+STRB_WIDTH:DATA_WIDTH+1];

axi_register_slice #
(
   .DATA_WIDTH              ( DATA_WIDTH+STRB_WIDTH+1         )
)
u0_axi_register_slice
(
   .clk                     ( USER_CLK                        ),
   .reset                   ( RESET                           ),
   .clk_en                  ( 1'b1                            ),

   .s_axi_valid             ( s_axi_valid_reg                 ),
   .s_axi_ready             ( s_axi_ready_reg                 ),
   .s_axi_data              ( s_axi_data_reg                  ),

   .m_axi_valid             ( m_axi_valid_reg                 ),
   .m_axi_ready             ( m_axi_ready_reg                 ),
   .m_axi_data              ( m_axi_data_reg                  )
);


assign s_axi_valid_reg0           = LL_OP_SRC_RDY;
assign LL_IP_DST_RDY_INT          = s_axi_ready_reg0;
assign s_axi_data_reg0            = {LL_TUSER_INT,LL_OP_DATA_INT};
assign m_axi_ready_reg0           = LL_IP_DST_RDY;
assign LL_OP_SRC_RDY_INT          = m_axi_valid_reg0;
assign LL_OP_DATA                 = m_axi_data_reg0[DATA_WIDTH-1:00];
assign LL_TUSER                   = m_axi_data_reg0[DATA_WIDTH+REM_WIDTH+1:DATA_WIDTH];

axi_register_slice #
(
   .DATA_WIDTH              ( DATA_WIDTH+REM_WIDTH+2          )
)
u1_axi_register_slice
(
   .clk                     ( USER_CLK                        ),
   .reset                   ( RESET                           ),
   .clk_en                  ( 1'b1                            ),

   .s_axi_valid             ( s_axi_valid_reg0                ),
   .s_axi_ready             ( s_axi_ready_reg0                ),
   .s_axi_data              ( s_axi_data_reg0                 ),

   .m_axi_valid             ( m_axi_valid_reg0                ),
   .m_axi_ready             ( m_axi_ready_reg0                ),
   .m_axi_data              ( m_axi_data_reg0                 )
);

    assign LL_IP_DST_RDY    = !LL_IP_DST_RDY_N;

generate
  genvar i;
  for (i=0; i<BC; i=i+1) begin: pdu
    assign LL_OP_DATA_INT[((BC-1-i)*8):((BC-1-i)*8)+7] = AXI4_S_IP_TX_TDATA_m[((BC-1-i)*8)+7:((BC-1-i)*8)];
  end
endgenerate

generate
  genvar j;
  for (j=0; j<STRB_WIDTH; j=j+1) begin: strb_pdu
    assign AXI4_S_IP_TX_TKEEP_i[j] = AXI4_S_IP_TX_TKEEP_m[j];
  end
endgenerate

    assign LL_OP_SRC_RDY    = AXI4_S_IP_TX_TVALID_m;
    assign LL_OP_EOF        = AXI4_S_IP_TX_TLAST_m;
 
  always @ (found_rem or AXI4_S_IP_TX_TKEEP_i)
  begin
    found_rem = 1'b0;
    LL_OP_REM_INT = {(STRB_WIDTH-1){1'b0}};
    for (i_rem = 0; i_rem < STRB_WIDTH; i_rem = i_rem + 1) begin
       if ((AXI4_S_IP_TX_TKEEP_i[i_rem] == 1'b0) && (found_rem == 1'b0)) begin
         LL_OP_REM_INT = i_rem;
         found_rem = 1'b1;
       end
    end
  end

    assign LL_OP_SOF        =  (( AXI4_S_IP_TX_TVALID_m && AXI4_S_IP_TX_TREADY_m && AXI4_S_IP_TX_TLAST_m ) ? ((new_pkt_r) ? 1'b0 : 1'b1) : (new_pkt && (!new_pkt_r)));
    assign LL_TUSER_INT     = {LL_OP_REM_INT,LL_OP_SOF,LL_OP_EOF};
 
    assign AXI4_S_IP_TX_TREADY_m = LL_IP_DST_RDY_INT;
    assign LL_OP_SRC_RDY_N = !LL_OP_SRC_RDY_INT;
    assign LL_OP_EOF_N = !LL_TUSER[(REM_WIDTH + 1)];
    assign LL_OP_REM = LL_TUSER[0:(REM_WIDTH - 1)];
    assign LL_OP_SOF_N  = ~ (LL_TUSER[REM_WIDTH]);

    assign new_pkt = ( AXI4_S_IP_TX_TVALID_m && AXI4_S_IP_TX_TREADY_m && AXI4_S_IP_TX_TLAST_m ) ? 1'b0 : ((AXI4_S_IP_TX_TVALID_m && AXI4_S_IP_TX_TREADY_m && !AXI4_S_IP_TX_TLAST_m ) ? 1'b1 : new_pkt_r);
    
    always @ (posedge USER_CLK)
    begin
      if(!CHANNEL_UP)
        new_pkt_r <=  1'b0;
      else
        new_pkt_r <=  new_pkt;
    end
 
endmodule
