
`timescale 1 ns / 10 ps
 module frame_check
 (
     // User Interface
     RX_D,  
     RX_REM,
     RX_SOF_N,
     RX_EOF_N,
     RX_SRC_RDY_N,  
     DATA_ERR_COUNT,
 
     //User Flow Control Interface
     UFC_RX_DATA,
     UFC_RX_REM,
     UFC_RX_SOF_N,
     UFC_RX_EOF_N,
     UFC_RX_SRC_RDY_N,
     UFC_ERR_COUNT,
     UFC_IN_PROGRESS,

     //User K Interface
     RX_USER_K_DATA,
     RX_USER_K_SRC_RDY_N,
     RX_USER_K_BLK_NO,
     USER_K_ERR_COUNT,
 
     // System Interface
     CHANNEL_UP,
     USER_CLK,    
     RESET4RX,     
     RESET
   
 );
 //*********************** Parameter Declarations************************
     parameter            LANE_WIDTH      = 1;
     parameter            LANE_DATA_WIDTH = (LANE_WIDTH*64);
     parameter            REM_BUS         = 5;
     parameter            DATA_WIDTH      = 8;
 
 //***********************************Port Declarations*******************************
     //PDU Interface
     input     [0:LANE_DATA_WIDTH-1]    RX_D;
     input     [0:REM_BUS-1]            RX_REM;
     input                              RX_SOF_N; 
     input                              RX_EOF_N; 
     input                              RX_SRC_RDY_N; 
     
     //System Interface
     input                              CHANNEL_UP; 
     input                              USER_CLK; 
     input                              RESET4RX;
     input                              RESET;  
     output    [0:DATA_WIDTH-1]         DATA_ERR_COUNT;
 
     //UFC Interface
     input     [0:LANE_DATA_WIDTH-1]    UFC_RX_DATA;
     input     [0:REM_BUS-1]            UFC_RX_REM;
     input                              UFC_RX_SOF_N;
     input                              UFC_RX_EOF_N;
     input                              UFC_RX_SRC_RDY_N;
     output    [0:DATA_WIDTH-1]         UFC_ERR_COUNT;
     input                              UFC_IN_PROGRESS;

     //USER K Interface
     input     [0:LANE_WIDTH*56-1]      RX_USER_K_DATA;
     input                              RX_USER_K_SRC_RDY_N;
     input     [0:LANE_WIDTH*4-1]       RX_USER_K_BLK_NO;
     output    [0:DATA_WIDTH-1]         USER_K_ERR_COUNT;
 
 //***************************Internal Register Declarations*************************** 
     reg     [0:LANE_DATA_WIDTH-1]     rx_d_reg;
     reg                               rx_src_rdy_n_reg; 
     reg       [0:DATA_WIDTH-1]        data_err_count_r; 
     reg       [0:LANE_WIDTH-1]        data_err_r;
 
     //PDU interface signals
     reg       [0:15]                   pdu_lfsr_r;
     wire      [LANE_DATA_WIDTH-1:0]    pdu_cmp_data_w;
     wire      [0:LANE_DATA_WIDTH-1]    pdu_cmp_data_w_r;
 
     //PDU interface signals
     reg       [0:DATA_WIDTH-1]         DATA_ERR_COUNT; 
///     reg       [0:15]                   pdu_lfsr_r;
     reg       [LANE_DATA_WIDTH-1:0]    pdu_cmp_data_r;
     reg       [0:LANE_DATA_WIDTH-1]    RX_D_R;  
     reg                                pdu_data_valid_r;
     reg                                pdu_in_frame_r;
     reg       [0:LANE_WIDTH-1]         pdu_err_detected_c=0;
     wire       [0:LANE_DATA_WIDTH-1]   pdu_cmp_data_r1;
     wire      [0:LANE_WIDTH-1]         data_err_c; 

     reg       [0:LANE_DATA_WIDTH-1]    RX_D_R2;
     reg       [0:REM_BUS-1]            RX_REM_R2;
     reg       [0:REM_BUS-1]            RX_REM_R3;
     reg                                RX_SOF_N_R2;
     reg                                RX_EOF_N_R2;
     reg                                RX_SRC_RDY_N_R2; 
 
     wire                               pdu_in_frame_c;
     wire      [0:LANE_DATA_WIDTH-1]    pdu_lfsr_concat_w;  
     wire                               pdu_data_valid_c;
 
     //UFC interface signals 
     reg       [0:15]                   ufc_lfsr_r;
     reg       [0:LANE_DATA_WIDTH-1]    ufc_cmp_data_r;
     reg       [0:LANE_DATA_WIDTH-1]    UFC_RX_D_R;  
     reg       [0:DATA_WIDTH-1]         UFC_ERR_COUNT;
     reg                                ufc_data_valid_r;
     reg                                ufc_in_frame_r;
     
     wire                               ufc_in_frame_c;
     wire      [0:LANE_DATA_WIDTH-1]    ufc_lfsr_concat_w;
     wire                               ufc_data_valid_c;
     reg       [0:15]                   ufc_err_detected_c;
     wire      [0:LANE_WIDTH - 1]       ufc_err_c; 
     wire                               resetUFC;
     
     reg                                ufc_in_progress_r = 'b0; 
     reg       [2:0]                    count4reset4rx = 'b0; 
     wire                               dlydRESET;

     //USER K interface signals
     reg                                user_k_err;
     reg       [0:DATA_WIDTH-1]         USER_K_ERR_COUNT;

     wire                               reset_i; 
     wire                               RESET_ii; 

     reg                                firstData_r;
     reg                                firstData_r2;

     wire                               firstDataChk;
     wire                               chk_data;
     reg                                firstDataChk_r;

 //*********************************Main Body of Code**********************************

       assign reset_i = RESET || RESET4RX || (!CHANNEL_UP);
     always @(posedge USER_CLK)
         if (RESET)
         	 count4reset4rx <=  'b0;
         else if ((count4reset4rx != 'b0) && ((ufc_in_progress_r && UFC_IN_PROGRESS )  != 'b0) && CHANNEL_UP)
         	 count4reset4rx <=  count4reset4rx - 1'b1;
         else if (RESET4RX && ((ufc_in_progress_r || UFC_IN_PROGRESS )  == 'b0) && CHANNEL_UP)
         	 count4reset4rx <=  3'b111;
    
     always @(posedge USER_CLK)
         	 ufc_in_progress_r <=  UFC_IN_PROGRESS;
    
    assign dlydRESET = RESET || ((count4reset4rx < 3'd7) && (count4reset4rx > 3'd0)); 

        assign resetUFC = (!CHANNEL_UP || count4reset4rx == 3'd0)? reset_i: dlydRESET; 


    assign RESET_ii = RESET || RESET4RX; 

  /*****************************PDU Data Genetration & Checking**********************/
     always @ (posedge USER_CLK)
       begin
         rx_d_reg         <=  RX_D;
         rx_src_rdy_n_reg <=  RX_SRC_RDY_N;
       end

     //Generate the PDU data using LFSR for data comparision
     always @ (posedge USER_CLK)
     if(reset_i)
       pdu_lfsr_r  <=    16'hD5E6;
     else if(pdu_data_valid_c)
       pdu_lfsr_r  <=    {!{pdu_lfsr_r[3]^pdu_lfsr_r[12]^pdu_lfsr_r[14]^pdu_lfsr_r[15]}, 
                            pdu_lfsr_r[0:14]};
 
     assign pdu_lfsr_concat_w = {LANE_WIDTH*4{pdu_lfsr_r}};

    always @ (posedge USER_CLK)
    begin
       RX_D_R2         <=  RX_D;
       RX_SRC_RDY_N_R2 <=  RX_SRC_RDY_N; 
    end

    always @ (posedge USER_CLK)
    begin
       RX_REM_R2       <=  RX_REM; 
       RX_REM_R3       <=  RX_REM;
       RX_SOF_N_R2     <=  RX_SOF_N; 
       RX_EOF_N_R2     <=  RX_EOF_N; 
    end
 
     //______________________________ Capture incoming data ___________________________    
 
     //PDU data is valid when RX_SRC_RDY_N_R2 is asserted

     assign pdu_data_valid_c    =   pdu_in_frame_c && !RX_SRC_RDY_N_R2;

    
     //PDU data is in a frame if it is a single cycle frame or a multi_cycle frame has started
     assign  pdu_in_frame_c  =   pdu_in_frame_r  ||  (!RX_SRC_RDY_N_R2 && !RX_SOF_N_R2);
 
     //Start a multicycle PDU frame when a frame starts without ending on the same cycle. End 
     //the frame when an RX_EOF_N_R2 is detected
     always @(posedge USER_CLK)
     if(RESET_ii)
       pdu_in_frame_r  <=      1'b0;
     else if(!pdu_in_frame_r && !RX_SOF_N_R2 && !RX_SRC_RDY_N_R2 && RX_EOF_N_R2)
       pdu_in_frame_r  <=      1'b1;
     else if(pdu_in_frame_r && !RX_SRC_RDY_N_R2 && !RX_EOF_N_R2)
       pdu_in_frame_r  <=      1'b0;

     always @ (posedge USER_CLK)
     begin 	       
       if((!RX_EOF_N_R2) && (!RX_SRC_RDY_N_R2))
       begin	 
         case(RX_REM_R2)
         3'd0 : RX_D_R <=   RX_D_R2;
         3'd1 : RX_D_R <=   {RX_D_R2[0:7],  56'b0};
         3'd2 : RX_D_R <=   {RX_D_R2[0:15], 48'b0};
         3'd3 : RX_D_R <=   {RX_D_R2[0:23], 40'b0};
         3'd4 : RX_D_R <=   {RX_D_R2[0:31], 32'b0};
         3'd5 : RX_D_R <=   {RX_D_R2[0:39], 24'b0};
         3'd6 : RX_D_R <=   {RX_D_R2[0:47], 16'b0};
         3'd7 : RX_D_R <=   {RX_D_R2[0:55], 8'b0 };
         default : RX_D_R  <=   RX_D_R2; 		 
 	endcase 	 
       end  
       else if(!RX_SRC_RDY_N_R2)
         RX_D_R          <=      RX_D_R2;
     end 
    
             //Calculate the expected PDU data 
     always @ (posedge USER_CLK)
     begin
       if(reset_i)
         pdu_cmp_data_r <=  {LANE_WIDTH*4{16'hD5E6}};
       else if(pdu_data_valid_c)
       begin		 
         case(RX_REM_R3)
         3'd0 : pdu_cmp_data_r  <=   pdu_lfsr_concat_w;
         3'd1 : pdu_cmp_data_r  <=   {pdu_lfsr_concat_w[0:7],   56'b0};
         3'd2 : pdu_cmp_data_r  <=   {pdu_lfsr_concat_w[0:15],  48'b0};
         3'd3 : pdu_cmp_data_r  <=   {pdu_lfsr_concat_w[0:23],  40'b0};
         3'd4 : pdu_cmp_data_r  <=   {pdu_lfsr_concat_w[0:31],  32'b0};
         3'd5 : pdu_cmp_data_r  <=   {pdu_lfsr_concat_w[0:39],  24'b0};
         3'd6 : pdu_cmp_data_r  <=   {pdu_lfsr_concat_w[0:47],  16'b0};
         3'd7 : pdu_cmp_data_r  <=   {pdu_lfsr_concat_w[0:55],  8'b0 };
         default : pdu_cmp_data_r <=   pdu_lfsr_concat_w; 		 
 	endcase 	 
       end
     end


 
     //PDU Data in the pdu_cmp_data_r register is valid only if it was valid when captured and had no error
     always @(posedge USER_CLK)
       if(reset_i)   
         pdu_data_valid_r    <=      1'b0;
       else
         pdu_data_valid_r    <=      pdu_data_valid_c && !pdu_err_detected_c;

     //
     assign pdu_cmp_data_r1  = pdu_cmp_data_r;
     assign pdu_cmp_data_w_r = pdu_cmp_data_w;

generate begin
    genvar i;
    for (i = 0; i < LANE_WIDTH; i = i + 1) begin : gen_data_err
      assign data_err_c[i] = ( pdu_data_valid_r && (RX_D_R[i*64:i*64+63] != pdu_cmp_data_r1[i*64:i*64+63]));         
    end
end
endgenerate 

     //An error is detected when LFSR generated PDU data from the pdu_cmp_data_r register, 
     //does not match valid data from the RX_D port
 always @(posedge USER_CLK)
     pdu_err_detected_c    <=      data_err_c;

 
     //Compare the incoming PDU data with calculated expected PDU data.
     //Increment the PDU ERR COUNTER if mismatch occurs
     //Stop the PDU ERR COUNTER once it reaches its max value
     always @ (posedge USER_CLK)
     begin	       
       if(RESET_ii)
         DATA_ERR_COUNT <=  8'b0;
       else if(!CHANNEL_UP)
         DATA_ERR_COUNT <=  8'b0;
       else if(&DATA_ERR_COUNT)
         DATA_ERR_COUNT <=  DATA_ERR_COUNT;
       else if(|pdu_err_detected_c)
         DATA_ERR_COUNT <=  DATA_ERR_COUNT + 1; 
       end 
 
  /*****************************UFC Data Genetration & Checking**********************/
 
     //Generate the expected UFC data using LFSR 
     always @ (posedge USER_CLK)
       if(resetUFC)
         ufc_lfsr_r  <=    16'h77D6;
       else if(ufc_data_valid_c)
         ufc_lfsr_r  <=    {!{ufc_lfsr_r[3]^ufc_lfsr_r[12]^ufc_lfsr_r[14]^ufc_lfsr_r[15]}, 
                        ufc_lfsr_r[0:14]};
 
     assign ufc_lfsr_concat_w = {LANE_WIDTH*4{ufc_lfsr_r}};
 
     //UFC data is valid when UFC_RX_SRC_RDY_N is asserted
     assign  ufc_data_valid_c    =   ufc_in_frame_c && !UFC_RX_SRC_RDY_N;
 
     //UFC data is in a frame if it is a single cycle frame or a multi_cycle frame has started
     assign  ufc_in_frame_c  =   ufc_in_frame_r  ||  (!UFC_RX_SRC_RDY_N && !UFC_RX_SOF_N);
 
     //Start a multicycle UFC frame when a frame starts without ending on the same cycle. End 
     //the frame when an UFC_RX_EOF_N is detected
     always @(posedge USER_CLK)
       if(resetUFC)
         ufc_in_frame_r  <=      1'b0;
       else if(!ufc_in_frame_r && !UFC_RX_SOF_N && !UFC_RX_SRC_RDY_N && UFC_RX_EOF_N)
         ufc_in_frame_r  <=      1'b1;
       else if(ufc_in_frame_r && !UFC_RX_SRC_RDY_N && !UFC_RX_EOF_N)
         ufc_in_frame_r  <=      1'b0;


     always @ (posedge USER_CLK)
     begin 	       
       if((!UFC_RX_EOF_N) && (!UFC_RX_SRC_RDY_N))
       begin	 
         case(UFC_RX_REM)
         3'd0 : UFC_RX_D_R <=    UFC_RX_DATA;
         3'd1 : UFC_RX_D_R <=    {UFC_RX_DATA[0:7],   56'b0};
         3'd2 : UFC_RX_D_R <=    {UFC_RX_DATA[0:15],  48'b0};
         3'd3 : UFC_RX_D_R <=    {UFC_RX_DATA[0:23],  40'b0};
         3'd4 : UFC_RX_D_R <=    {UFC_RX_DATA[0:31],  32'b0};
         3'd5 : UFC_RX_D_R <=    {UFC_RX_DATA[0:39],  24'b0};
         3'd6 : UFC_RX_D_R <=    {UFC_RX_DATA[0:47],  16'b0};
         3'd7 : UFC_RX_D_R <=    {UFC_RX_DATA[0:55],  8'b0};
         default : UFC_RX_D_R <=   UFC_RX_DATA; 		 
 	endcase 	 
       end  
       else if(!UFC_RX_SRC_RDY_N)
         UFC_RX_D_R <=   UFC_RX_DATA;
     end 
 
     //Calculate the expected UFC data for comparision
     always @ (posedge USER_CLK)
     begin
       if(resetUFC)
         ufc_cmp_data_r <=  {LANE_WIDTH*4{16'h77D6}};		   
       else if(ufc_data_valid_c)  
       begin		 
         case(UFC_RX_REM)
         3'd0 : ufc_cmp_data_r <=    ufc_lfsr_concat_w;
         3'd1 : ufc_cmp_data_r <=    {ufc_lfsr_concat_w[0:7],   56'b0 };
         3'd2 : ufc_cmp_data_r <=    {ufc_lfsr_concat_w[0:15],  48'b0 };
         3'd3 : ufc_cmp_data_r <=    {ufc_lfsr_concat_w[0:23],  40'b0 };
         3'd4 : ufc_cmp_data_r <=    {ufc_lfsr_concat_w[0:31],  32'b0 };
         3'd5 : ufc_cmp_data_r <=    {ufc_lfsr_concat_w[0:39],  24'b0 };
         3'd6 : ufc_cmp_data_r <=    {ufc_lfsr_concat_w[0:47],  16'b0 };
         3'd7 : ufc_cmp_data_r <=    {ufc_lfsr_concat_w[0:55],  8'b0  };
         default : ufc_cmp_data_r <=  ufc_lfsr_concat_w;
         endcase 	 
       end
     end


     //UFC Data in the ufc_cmp_data_r register is valid only if it was valid when captured and had no error
     always @(posedge USER_CLK)
       if(resetUFC)   
         ufc_data_valid_r    <=      1'b0;
       else        
         ufc_data_valid_r    <=      ufc_data_valid_c  && !ufc_err_detected_c;


generate begin
    genvar i;
    for (i = 0; i < LANE_WIDTH; i = i + 1) begin : gen_ufc
      assign ufc_err_c[i] = ( ufc_data_valid_r && (UFC_RX_D_R[i*64:i*64+63] != ufc_cmp_data_r[i*64:i*64+63]));         
    end
end
endgenerate
   
     //An error is detected when valid UFC data from the ufc_cmp_data_r register, 
     //does not match valid data from the UFC_RX_DATA port
 always @(posedge USER_CLK)
        if (resetUFC)
        	ufc_err_detected_c    <=      1'b0;
        else
        	ufc_err_detected_c    <=      |ufc_err_c;
 
     //Compare the incoming UFC data with calculated expected UFC data.
     //Increment the UFC ERR COUNTER if mismatch occurs
     //Stop the UFC ERR COUNTER once it reaches its max value
     always @ (posedge USER_CLK)
     begin	       
       if(resetUFC)
         UFC_ERR_COUNT <=  8'b0;
       else if(&UFC_ERR_COUNT)
         UFC_ERR_COUNT <=  UFC_ERR_COUNT;
       else if(|ufc_err_detected_c)
         UFC_ERR_COUNT <=  (UFC_ERR_COUNT + 1);   	 
       end

 
  /*****************************USER K Data Checking**********************/
 
     //Compare the USER K Data with expected USER K Data
     //Generate the pulse if mismatch occurs
     always @ (posedge USER_CLK)
     begin
       if(RX_USER_K_SRC_RDY_N) begin
          user_k_err <= 1'b0; 
       end 
       else begin
         case(RX_USER_K_BLK_NO[0:3])
             4'b0000 : user_k_err <=  (RX_USER_K_DATA != {LANE_WIDTH{16'hAB40, 40'h0}});
             4'b0001 : user_k_err <=  (RX_USER_K_DATA != {LANE_WIDTH{16'hCD90, 40'h0}});
             4'b0010 : user_k_err <=  (RX_USER_K_DATA != {LANE_WIDTH{16'h1230, 40'h0}});
             4'b0011 : user_k_err <=  (RX_USER_K_DATA != {LANE_WIDTH{16'h5678, 40'h0}});
             4'b0100 : user_k_err <=  (RX_USER_K_DATA != {LANE_WIDTH{16'h1EBC, 40'h0}});
             4'b0101 : user_k_err <=  (RX_USER_K_DATA != {LANE_WIDTH{16'hDEF0, 40'h0}});
             4'b0110 : user_k_err <=  (RX_USER_K_DATA != {LANE_WIDTH{16'h2D7F, 40'h0}});
             4'b0111 : user_k_err <=  (RX_USER_K_DATA != {LANE_WIDTH{16'h78FD, 40'h0}});
             4'b1000 : user_k_err <=  (RX_USER_K_DATA != {LANE_WIDTH{16'hAAE1, 40'h0}});
             default : user_k_err <=  1'b0;
          endcase
       end
     end
 
     //Compare the incoming USER K data with calculated expected USER K data.
     //Increment the USER K ERR COUNTER if mismatch occurs
     //Stop the USER K ERR COUNTER once it reaches its max value
     always @ (posedge USER_CLK)
     begin
       if(reset_i)
           USER_K_ERR_COUNT <=  8'h0;
       else if(user_k_err && !(&USER_K_ERR_COUNT)) 
           USER_K_ERR_COUNT <=  USER_K_ERR_COUNT + 1'b1;
     end


 endmodule           
