
`timescale 1 ns/1 ps

 module ll_to_axi_reg #
 (
    parameter            DATA_WIDTH         = 16, // DATA bus width
    parameter            STRB_WIDTH         = 2, // STROBE bus width
    parameter            USE_4_NFC          = 0, // 0 => PDU, 1 => NFC, 2 => UFC and 3 => USER K
    parameter            BC                 =  DATA_WIDTH>>3, //Byte count
    parameter            REM_WIDTH          = 1 // REM bus width    
 )
 ( 
    // LocalLink input Interface
    LL_IP_DATA,
    LL_IP_SOF_N,
    LL_IP_EOF_N,
    LL_IP_REM,
    LL_IP_SRC_RDY_N,
    LL_OP_DST_RDY_N,
 
    // AXI4-S output signals
    AXI4_S_OP_TVALID,
    AXI4_S_OP_TDATA,
    AXI4_S_OP_TKEEP,
    AXI4_S_OP_TLAST,

    USER_CLK,
    RESET,

    AXI4_S_IP_TREADY 
 );

 //***********************************Port Declarations******************************* 
    // AXI4-Stream TX Interface
    output     [(DATA_WIDTH-1):0]     AXI4_S_OP_TDATA;
    output     [(STRB_WIDTH-1):0]     AXI4_S_OP_TKEEP;

    output                            AXI4_S_OP_TVALID;
    output                            AXI4_S_OP_TLAST;
    input                             AXI4_S_IP_TREADY;

    input                             USER_CLK;
    input                             RESET;

    // LocalLink TX Interface
    input      [0:(DATA_WIDTH-1)]     LL_IP_DATA;
    input      [0:(REM_WIDTH-1)]      LL_IP_REM;
    input                             LL_IP_SOF_N;
    input                             LL_IP_EOF_N;
    input                             LL_IP_SRC_RDY_N;
    output                            LL_OP_DST_RDY_N;

 //***********************************Signale Declarations******************************* 
    wire [0:(STRB_WIDTH-1)]           AXI4_S_OP_TKEEP_i;

    wire [0:(DATA_WIDTH-1)]          AXI4_S_OP_TDATA_INT;
    wire [0:(STRB_WIDTH-1)]          AXI4_S_OP_TKEEP_INT;
    wire                             AXI4_S_OP_TVALID_INT;
    wire                             AXI4_S_OP_TLAST_INT;
    wire                             AXI4_S_IP_TREADY_INT;

    wire                             reset_n;
    wire     [0:(DATA_WIDTH-1)]      AXI4_S_OP_TDATA_i;

    wire                             s_axi_valid_reg;
    wire                             s_axi_ready_reg;
    wire [DATA_WIDTH+STRB_WIDTH:0]   s_axi_data_reg; 
    
    wire                             m_axi_valid_reg;
    wire                             m_axi_ready_reg;
    wire [DATA_WIDTH+STRB_WIDTH:0]   m_axi_data_reg; 
 
 //*********************************Main Body of Code********************************** 
generate
 if(USE_4_NFC==0)
 begin
  genvar i;
  for (i=0; i<BC; i=i+1) begin: pdu
    assign AXI4_S_OP_TDATA[((BC-1-i)*8)+7:((BC-1-i)*8)] = AXI4_S_OP_TDATA_i[((BC-1-i)*8):((BC-1-i)*8)+7];
end
end
endgenerate

generate
 if(USE_4_NFC==1)
 begin
  genvar i;
  for (i=0; i<BC; i=i+1) begin: nfc
    assign AXI4_S_OP_TDATA[(i*8)+7:(i*8)] = AXI4_S_OP_TDATA_i[((BC-1-i)*8):((BC-1-i)*8)+7];
end
end
endgenerate

generate
 if(USE_4_NFC==2)
 begin
  genvar i;
  for (i=0; i<BC; i=i+1) begin: ufc
    assign AXI4_S_OP_TDATA[((BC-1-i)*8)+7:((BC-1-i)*8)] = AXI4_S_OP_TDATA_i[((BC-1-i)*8):((BC-1-i)*8)+7];
end
end
endgenerate

generate
 if(USE_4_NFC==3)
 begin
  genvar i;
  for (i=0; i<BC; i=i+1) begin: userk
    assign AXI4_S_OP_TDATA[((BC-1-i)*8)+7:((BC-1-i)*8)] = AXI4_S_OP_TDATA_i[((BC-1-i)*8):((BC-1-i)*8)+7];
end
end
endgenerate


generate
 if(USE_4_NFC==2)
 begin
  genvar j;
  for (j=0; j<REM_WIDTH; j=j+1) begin: strb_ufc
    assign AXI4_S_OP_TKEEP[j] = LL_IP_REM[(REM_WIDTH-1)-j];
 end
 end
endgenerate

generate
 if(USE_4_NFC==0)
 begin
  genvar k;
  for (k=0; k<STRB_WIDTH; k=k+1) begin: strb_pdu
    assign AXI4_S_OP_TKEEP[k] = AXI4_S_OP_TKEEP_i[k];
 end
 end
endgenerate


    assign AXI4_S_OP_TDATA_INT  = LL_IP_DATA;
    assign AXI4_S_OP_TVALID_INT = !LL_IP_SRC_RDY_N;

    assign AXI4_S_OP_TLAST_INT  = !LL_IP_EOF_N;
 
    assign AXI4_S_OP_TKEEP_INT  = (LL_IP_REM == {REM_WIDTH{1'b0}})? ({STRB_WIDTH{1'b1}}) :
                              (~({STRB_WIDTH{1'b1}}>>(LL_IP_REM)));
 
    assign LL_OP_DST_RDY_N  = !AXI4_S_IP_TREADY_INT;


assign s_axi_valid_reg        = AXI4_S_OP_TVALID_INT;
assign AXI4_S_IP_TREADY_INT   = s_axi_ready_reg;
assign s_axi_data_reg         = {AXI4_S_OP_TKEEP_INT,AXI4_S_OP_TLAST_INT,AXI4_S_OP_TDATA_INT};
assign m_axi_ready_reg        = AXI4_S_IP_TREADY;
assign AXI4_S_OP_TVALID       = m_axi_valid_reg;
assign AXI4_S_OP_TDATA_i      = m_axi_data_reg[DATA_WIDTH-1:00];
assign AXI4_S_OP_TLAST        = m_axi_data_reg[DATA_WIDTH];
assign AXI4_S_OP_TKEEP_i      = m_axi_data_reg[DATA_WIDTH+STRB_WIDTH:DATA_WIDTH+1];

axi_register_slice #
(
   .DATA_WIDTH              ( DATA_WIDTH+STRB_WIDTH+1         )
)
u_axi_register_slice
(
   .clk                     ( USER_CLK                        ),
   .reset                   ( RESET                           ),
   .clk_en                  ( 1'b1                            ),

   .s_axi_valid             ( s_axi_valid_reg                 ),
   .s_axi_ready             ( s_axi_ready_reg                 ),
   .s_axi_data              ( s_axi_data_reg                  ),

   .m_axi_valid             ( m_axi_valid_reg                 ),
   .m_axi_ready             ( m_axi_ready_reg                 ),
   .m_axi_data              ( m_axi_data_reg                  )
);

 endmodule
