
`timescale 1 ns / 10 ps

module top  
(
    input           osc_clk_i, // 50M
    input           resetn_i,///SW1
    output          button_sw2_o,//SW2
    output          button_sw3_o,//SW3
    output          button_sw4_o,//SW4
	
    output          sfp1_tx_disable_o,
    output          sfp2_tx_disable_o,
    input           sfp1_rx_los_i,
    
    input           rx_i,
    output          tx_o,
    
    output [3:0]    test_o,
    output [3:0]    led_o      

);

//***********************************Parameter****************************************//
parameter       LANE_WIDTH             = 1;
parameter       DATA_WIDTH             = 64*LANE_WIDTH;
parameter       KEEP_WIDTH             = 8*LANE_WIDTH;
parameter       USERK_WIDTH            = 56*LANE_WIDTH;
parameter       USERK_BLK_WIDTH        = 4*LANE_WIDTH;

parameter       REM_WIDTH              = (LANE_WIDTH == 1) ? 3 : (LANE_WIDTH == 2) ? 4 : 
                                         ((LANE_WIDTH == 3) | (LANE_WIDTH == 4)) ? 5 : 6;

//***********************************Signal Declarations*******************************//
// --------- user interface TX Data interface  
wire [64*LANE_WIDTH-1:0]              user_tx_data/* synthesis syn_keep="1" */;
wire                                  user_tx_valid/* synthesis syn_keep="1" */;
wire                                  user_tx_ready/* synthesis syn_keep="1" */;

wire [8*LANE_WIDTH-1:0]               user_tx_strb/* synthesis syn_keep="1" */;
wire                                  user_tx_last/* synthesis syn_keep="1" */;

// --------- user interface RX Data interface  
wire [64*LANE_WIDTH-1:0]              user_rx_data/* synthesis syn_keep="1" */;
wire                                  user_rx_valid/* synthesis syn_keep="1" */;
wire [8*LANE_WIDTH-1:0]               user_rx_strb/* synthesis syn_keep="1" */;
wire                                  user_rx_last/* synthesis syn_keep="1" */;

// --------- User Flow Control interface  
wire                                  ufc_tx_req/* synthesis syn_keep="1" */;
wire [7:0]                            ufc_tx_ms/* synthesis syn_keep="1" */;
wire [64*LANE_WIDTH-1:0]              ufc_tx_data/* synthesis syn_keep="1" */;
wire                                  ufc_tx_valid/* synthesis syn_keep="1" */;
wire                                  ufc_tx_ready/* synthesis syn_keep="1" */;

wire [64*LANE_WIDTH-1:0]              ufc_rx_data/* synthesis syn_keep="1" */;
wire [8*LANE_WIDTH-1:0]               ufc_rx_strb/* synthesis syn_keep="1" */;
wire                                  ufc_rx_last/* synthesis syn_keep="1" */;
wire                                  ufc_rx_valid/* synthesis syn_keep="1" */;
wire                                  ufc_in_progress/* synthesis syn_keep="1" */;

// --------- user interface TX K-Core interface 
wire [64*LANE_WIDTH-1:0]              user_tx_k_data/* synthesis syn_keep="1" */;
wire                                  user_tx_k_valid/* synthesis syn_keep="1" */;
wire                                  user_tx_k_ready/* synthesis syn_keep="1" */;

// --------- user interface RX K-Core interface 
wire [64*LANE_WIDTH-1:0]              user_rx_k_data/* synthesis syn_keep="1" */;
wire                                  user_rx_k_valid/* synthesis syn_keep="1" */;

// --------- Status interface  
wire                                  hard_err;
wire                                  soft_err;
wire                                  channel_up;
wire [LANE_WIDTH-1:0]                 lane_up;
wire                                  tx_channel_up;
wire                                  rx_channel_up;

wire                                  crc_pass_fail_n;
wire                                  crc_valid;

//-------- User define serdes control
wire                                  tx_sys_reset;
wire                                  reset_tx;

wire                                  reset_rx;
wire                                  rx_sys_reset;

wire [0:64*LANE_WIDTH-1]              tx_d;        
wire                                  tx_sof_n;    
wire                                  tx_eof_n;    
wire [0:REM_WIDTH-1]                  tx_rem;     
wire                                  tx_src_rdy_n;
wire                                  tx_dst_rdy_n;

wire [0:64*LANE_WIDTH-1]              ufc_ll_tx_data;     
wire [0:7]                            ufc_ll_tx_ms;       
wire                                  ufc_ll_tx_src_rdy_n;
wire                                  ufc_ll_tx_dst_rdy_n;
wire                                  ufc_tx_req_n;


wire [0:64*LANE_WIDTH-1]              user_k_ll_data;  
wire                                  user_k_src_rdy_n;
wire                                  user_k_dst_rdy_n;

wire [0:4-1]                          user_k_blk_no;
wire [0:56*LANE_WIDTH-1]              user_k_data;

wire [0:DATA_WIDTH-1]                 rx_d;          
wire                                  rx_sof_n;    
wire                                  rx_eof_n;    
wire [0:REM_WIDTH-1]                  rx_rem;   
wire                                  rx_src_rdy_n;

wire [0:DATA_WIDTH-1]                 ufc_ll_rx_data;       
wire                                  ufc_ll_rx_sof_n;     
wire                                  ufc_ll_rx_eof_n;     
wire [0:REM_WIDTH-1]                  ufc_ll_rx_rem;       
wire                                  ufc_ll_rx_src_rdy_n; 

wire [0:56*LANE_WIDTH-1]              ll_rx_user_k_data;     
wire                                  ll_rx_user_k_src_rdy_n;
wire [0:4*LANE_WIDTH-1]               ll_rx_user_k_blk_no;
wire [0:64*LANE_WIDTH-1]              rx_user_k_data;

wire  [7:0]                           user_k_err_count;
wire  [7:0]                           ufc_err_count;
reg   [7:0]                           crc_err_count;
wire  [7:0]                           data_err_count;

wire                                  gt_pma_rstn;
wire [LANE_WIDTH-1:0]                 gt_pcs_tx_clk;
///wire [LANE_WIDTH-1:0]                 gt_pcs_rx_clk/* synthesis syn_keep=1 */;
wire                                  gt_pll_ok;  

wire                                  gt_pcs_tx_rst;

wire                                  gt_pcs_rx_rst;
wire [LANE_WIDTH-1:0]                 gt_rx_align_link;
wire [LANE_WIDTH-1:0]                 gt_rx_pma_lock;  
wire [LANE_WIDTH-1:0]                 gt_rx_block_sync;
wire [LANE_WIDTH-1:0]                 gt_rx_charerr;   

wire                                  sys_clk/* synthesis syn_keep=1 */;
wire                                  sys_rst;
wire                                  cfg_clk;
wire                                  cfg_pll_lock;
wire                                  cfg_rst;

wire                                  reg_rst;
wire                                  sys_reset_gen;

wire  [17:0]                          apb0_addr/* synthesis syn_keep=1 */;
wire                                  apb0_sel/* synthesis syn_keep=1 */;
wire                                  apb0_ena/* synthesis syn_keep=1 */;
wire                                  apb0_wr/* synthesis syn_keep=1 */;
wire  [31:0]                          apb0_rdata/* synthesis syn_keep=1 */;
wire  [31:0]                          apb0_wdata/* synthesis syn_keep=1 */;
wire                                  apb0_rdy/* synthesis syn_keep=1 */;
wire  [3:0]                           apb0_strb;

wire                                  local_wren/* synthesis syn_keep=1 */;
wire  [15:0]                          local_addr/* synthesis syn_keep=1 */;
wire                                  local_rden/* synthesis syn_keep=1 */;
wire  [31:0]                          local_wdat/* synthesis syn_keep=1 */;
wire  [31:0]                          local_rdat/* synthesis syn_keep=1 */;
wire  [3 :0]                          local_strb/* synthesis syn_keep=1 */;
wire                                  local_rdat_vld/* synthesis syn_keep=1 */;
wire                                  local_wdat_rdy/* synthesis syn_keep=1 */;

wire [31:0] reg0x0000;wire [31:0] mon0x0010;
wire [31:0] reg0x0001;wire [31:0] mon0x0011;
wire [31:0] reg0x0002;wire [31:0] mon0x0012;
wire [31:0] reg0x0003;wire [31:0] mon0x0013;
wire [31:0] reg0x0004;wire [31:0] mon0x0014;
wire [31:0] reg0x0005;wire [31:0] mon0x0015;
wire [31:0] reg0x0006;wire [31:0] mon0x0016;
wire [31:0] reg0x0007;wire [31:0] mon0x0017;
wire [31:0] reg0x0008;wire [31:0] mon0x0018;
wire [31:0] reg0x0009;wire [31:0] mon0x0019;
wire [31:0] reg0x000A;wire [31:0] mon0x001A;
wire [31:0] reg0x000B;wire [31:0] mon0x001B;
wire [31:0] reg0x000C;wire [31:0] mon0x001C;
wire [31:0] reg0x000D;wire [31:0] mon0x001D;
wire [31:0] reg0x000E;wire [31:0] mon0x001E;
wire [31:0] reg0x000F;wire [31:0] mon0x001F;

reg                                   test_pass;
reg [7:0]                             soft_err_cnt;
reg [7:0]                             hard_err_cnt;
reg [7:0]                             channel_resync_cnt;
reg [7:0]                             lane_resync_cnt;
reg [7:0]                             data_err_count_1d;
reg [7:0]                             crc_err_count_1d;
reg [7:0]                             ufc_err_count_1d;
reg [7:0]                             user_k_err_count_1d;
reg                                   soft_err_1d;   
reg                                   hard_err_1d;   
reg                                   channel_up_1d; 
reg                                   all_lane_up_1d;
reg                                   channel_up_2d; 
reg                                   all_lane_up_2d;
reg                                   tx_soft_err_1d;   
reg                                   tx_hard_err_1d;   
reg                                   tx_channel_up_1d; 
reg                                   tx_all_lane_up_1d;
reg                                   tx_channel_up_2d; 
reg                                   tx_all_lane_up_2d;
reg                                   rx_soft_err_1d;   
reg                                   rx_hard_err_1d;   
reg                                   rx_channel_up_1d; 
reg                                   rx_all_lane_up_1d;
reg                                   rx_channel_up_2d; 
reg                                   rx_all_lane_up_2d;
reg [7:0]                             tx_soft_err_cnt;
reg [7:0]                             tx_hard_err_cnt;
reg [7:0]                             tx_channel_resync_cnt;
reg [7:0]                             tx_lane_resync_cnt;
reg [7:0]                             rx_soft_err_cnt;
reg [7:0]                             rx_hard_err_cnt;
reg [7:0]                             rx_channel_resync_cnt;
reg [7:0]                             rx_lane_resync_cnt;

wire [31:0]                           cfg_tx_init_counter;
reg [7:0]                             gt_err_cnt;
reg [7:0]                             gt_block_resync_cnt;
reg                                   gt_block_sync_1d;
reg                                   gt_block_sync_2d;

//**************************************process**********************************//
assign sfp1_tx_disable_o = 1'b0;
assign sfp2_tx_disable_o = 1'b0;
assign test_o            = 4'd0;
assign led_o[0]          = cfg_pll_lock;
assign led_o[1]          = gt_pll_ok;
assign led_o[2]          = `ifdef TX_EN
                           tx_channel_up
                           `else
                           1'b1
                           `endif
                           &
                           `ifdef RX_EN
                           rx_channel_up
                           `else
                           1'b1
                           `endif
                           ;
assign led_o[3]          = test_pass;

assign sys_clk = gt_pcs_tx_clk[0];

Gowin_PLL u_Gowin_PLL
(
    .lock                                  ( cfg_pll_lock                     ), //output lock
    .clkout0                               ( cfg_clk                          ), //output clkout0
    .clkin                                 ( osc_clk_i                        )  //input clkin
);                                                                           

reset_gen u1_reset_gen                                                       
(                                                                            
    .i_clk1                                ( cfg_clk                          ),
    .i_lock                                ( cfg_pll_lock                     ),
    .o_rst1                                ( cfg_rst                          )
);                                                                           

assign sys_reset_gen = cfg_pll_lock & gt_pll_ok & !reg_rst & resetn_i;

reset_gen u2_reset_gen                                                       
(                                                                            
    .i_clk1                                ( sys_clk                          ),
    .i_lock                                ( sys_reset_gen                    ),
    .o_rst1                                ( sys_rst                          )
);

Uart_to_Bus_Top u_uart_bus_top
(
	.rst_n_i                               ( !cfg_rst                         ),                        
	.clk_i                                 ( cfg_clk                          ),                
	.uart_rx_led_o                         (                                  ),
	.uart_tx_led_o                         (                                  ),
	.uart_rx_i                             ( rx_i                             ),
	.uart_tx_o                             ( tx_o                             ),
    .apb0_addr_o                           ( apb0_addr                        ),
    .apb0_sel_o                            ( apb0_sel                         ),
    .apb0_ena_o                            ( apb0_ena                         ),
    .apb0_wr_o                             ( apb0_wr                          ),
    .apb0_rdata_i                          ( apb0_rdata                       ),
    .apb0_wdata_o                          ( apb0_wdata                       ),
    .apb0_rdy_i                            ( apb0_rdy                         ),
    .apb0_strb_o                           ( apb0_strb                        )  
);

apb2local u_apb2local
(
	.cfg_clk_i                             ( cfg_clk                          ),
	.cfg_rstn_i                            ( !cfg_rst                         ),
	.cfg_addr_i                            ( apb0_addr                        ),
	.cfg_sel_i                             ( apb0_sel                         ),
	.cfg_ena_i                             ( apb0_ena                         ),
	.cfg_wr_i                              ( apb0_wr                          ),
	.cfg_rdata_o                           ( apb0_rdata                       ),
	.cfg_wdata_i                           ( apb0_wdata                       ),
	.cfg_rdy_o                             ( apb0_rdy                         ),
	.cfg_strb_i                            ( apb0_strb                        ),

	.local_wren_o                          ( local_wren                       ),
	.local_addr_o                          ( local_addr                       ),
	.local_strb_o                          ( local_strb                       ),
	.local_rden_o                          ( local_rden                       ),
	.local_wdat_o                          ( local_wdat                       ),
	.local_rdat_i                          ( local_rdat                       ),
	.local_rdat_vld_i                      ( local_rdat_vld                   ),
	.local_wdat_rdy_i                      ( local_wdat_rdy                   )
);

local2reg u_local2reg
(

	.cfg_clk_i                             ( cfg_clk                          ),
	.cfg_rstn_i                            ( !cfg_rst                         ),
	.local_wren_i                          ( local_wren                       ),
	.local_addr_i                          ( local_addr                       ),
	.local_strb_i                          ( 4'hf                             ),	
	.local_rden_i                          ( local_rden                       ),
	.local_wdat_i                          ( local_wdat                       ),
	.local_rdat_o                          ( local_rdat                       ),
	.local_rdat_vld_o                      ( local_rdat_vld                   ),
	.local_wdat_rdy_o                      ( local_wdat_rdy                   ),	
 
	.reg0x0000(reg0x0000),.mon0x0010(mon0x0010),
	.reg0x0001(reg0x0001),.mon0x0011(mon0x0011),
	.reg0x0002(reg0x0002),.mon0x0012(mon0x0012),
    .reg0x0003(reg0x0003),.mon0x0013(mon0x0013),
    .reg0x0004(reg0x0004),.mon0x0014(mon0x0014),
    .reg0x0005(reg0x0005),.mon0x0015(mon0x0015),
    .reg0x0006(reg0x0006),.mon0x0016(mon0x0016),
    .reg0x0007(reg0x0007),.mon0x0017(mon0x0017),
    .reg0x0008(reg0x0008),.mon0x0018(mon0x0018),
    .reg0x0009(reg0x0009),.mon0x0019(mon0x0019),
    .reg0x000A(reg0x000A),.mon0x001A(mon0x001A),
    .reg0x000B(reg0x000B),.mon0x001B(mon0x001B),
    .reg0x000C(reg0x000C),.mon0x001C(mon0x001C),
    .reg0x000D(reg0x000D),.mon0x001D(mon0x001D),
    .reg0x000E(reg0x000E),.mon0x001E(mon0x001E),
    .reg0x000F(reg0x000F),.mon0x001F(mon0x001F)   		
);

assign mon0x0010[0]               = gt_pll_ok;
            
assign mon0x0010[1]               = soft_err;
assign mon0x0010[2]               = channel_up;
assign mon0x0010[3]               = hard_err;
assign mon0x0010[LANE_WIDTH+3:4]  = lane_up;
assign mon0x0013                  = {soft_err_cnt,hard_err_cnt,channel_resync_cnt,lane_resync_cnt};

assign mon0x0011[LANE_WIDTH-1:0]  = gt_rx_pma_lock;
assign mon0x0011[LANE_WIDTH+7:8]  = gt_rx_align_link;
assign mon0x0015                  = {gt_rx_block_sync,gt_block_resync_cnt,gt_err_cnt};

assign mon0x0012                  = {user_k_err_count,ufc_err_count,crc_err_count,data_err_count};

assign mon0x001E                  = 32'h20231208;
assign mon0x001F                  = 32'h00000000;

assign gt_pma_rstn                = !reg0x0001[0] & (!cfg_rst) & gt_pll_ok;
assign gt_pcs_tx_rst              = reg0x0001[1] | cfg_rst | (!gt_pll_ok);
assign gt_pcs_rx_rst              = reg0x0001[2] | cfg_rst | (!gt_pll_ok);
assign reg_rst                    = reg0x0001[3];

assign tx_channel_up          = channel_up;
assign rx_channel_up          = channel_up;
assign reset_tx               = tx_sys_reset | (!channel_up);
assign reset_rx               = rx_sys_reset | (!channel_up);

SerDes_Top u_SerDes_Top
(
    // --------- Clock & Reset
    .RoraLink_64B66B_Top_sys_rst_i             ( sys_rst                          ),
    .RoraLink_64B66B_Top_sys_clk_i             ( sys_clk                          ),
    .RoraLink_64B66B_Top_init_clk_i            ( cfg_clk                          ),

    // --------- user interface TX Data interface  
    .RoraLink_64B66B_Top_user_tx_data_i        ( user_tx_data                     ), 
    .RoraLink_64B66B_Top_user_tx_valid_i       ( user_tx_valid                    ),
    .RoraLink_64B66B_Top_user_tx_ready_o       ( user_tx_ready                    ),                          
    .RoraLink_64B66B_Top_user_tx_strb_i        ( user_tx_strb                     ),
    .RoraLink_64B66B_Top_user_tx_last_i        ( user_tx_last                     ),

    // --------- user interface RX Data interface  
    .RoraLink_64B66B_Top_user_rx_data_o        ( user_rx_data                     ),
    .RoraLink_64B66B_Top_user_rx_valid_o       ( user_rx_valid                    ),                           
    .RoraLink_64B66B_Top_user_rx_strb_o        ( user_rx_strb                     ),
    .RoraLink_64B66B_Top_user_rx_last_o        ( user_rx_last                     ),

    // --------- User Flow Control interface  
    .RoraLink_64B66B_Top_ufc_tx_req_i          ( ufc_tx_req                       ),
    .RoraLink_64B66B_Top_ufc_tx_ms_i           ( ufc_tx_ms                        ),
    .RoraLink_64B66B_Top_ufc_tx_data_i         ( ufc_tx_data                      ),
    .RoraLink_64B66B_Top_ufc_tx_valid_i        ( ufc_tx_valid                     ),
    .RoraLink_64B66B_Top_ufc_tx_ready_o        ( ufc_tx_ready                     ),                                      

    .RoraLink_64B66B_Top_ufc_rx_data_o         ( ufc_rx_data                      ),
    .RoraLink_64B66B_Top_ufc_rx_strb_o         ( ufc_rx_strb                      ),
    .RoraLink_64B66B_Top_ufc_rx_last_o         ( ufc_rx_last                      ),
    .RoraLink_64B66B_Top_ufc_rx_valid_o        ( ufc_rx_valid                     ),
    .RoraLink_64B66B_Top_ufc_in_progress_o     ( ufc_in_progress                  ),

    // --------- user interface TX K-Core interface 
    .RoraLink_64B66B_Top_user_tx_k_data_i      ( user_tx_k_data                   ),
    .RoraLink_64B66B_Top_user_tx_k_valid_i     ( user_tx_k_valid                  ),
    .RoraLink_64B66B_Top_user_tx_k_ready_o     ( user_tx_k_ready                  ),

    // --------- user interface RX K-Core interface 
    .RoraLink_64B66B_Top_user_rx_k_data_o      ( user_rx_k_data                   ),
    .RoraLink_64B66B_Top_user_rx_k_valid_o     ( user_rx_k_valid                  ),

    // --------- Status interface  
    .RoraLink_64B66B_Top_hard_err_o            ( hard_err                         ),
    .RoraLink_64B66B_Top_soft_err_o            ( soft_err                         ),
    .RoraLink_64B66B_Top_channel_up_o          ( channel_up                       ),
    .RoraLink_64B66B_Top_lane_up_o             ( lane_up                          ),

    .RoraLink_64B66B_Top_crc_pass_fail_n_o     ( crc_pass_fail_n                  ),
    .RoraLink_64B66B_Top_crc_valid_o           ( crc_valid                        ),
    
    //-------- User define serdes control
    .RoraLink_64B66B_Top_gt_pcs_tx_rst_i       ( gt_pcs_tx_rst                    ),
    .RoraLink_64B66B_Top_gt_tx_fifo_wrusewd_o  (                                  ),

    .RoraLink_64B66B_Top_gt_rx_block_sync_o    ( gt_rx_block_sync                 ),
    .RoraLink_64B66B_Top_gt_rx_charerr_o       ( gt_rx_charerr                    ),
    .RoraLink_64B66B_Top_gt_rx_align_link_o    ( gt_rx_align_link                 ),
    .RoraLink_64B66B_Top_gt_rx_pma_lock_o      ( gt_rx_pma_lock                   ),
    .RoraLink_64B66B_Top_gt_pcs_rx_clk_o       (                                  ),///gt_pcs_rx_clk                    ),
    .RoraLink_64B66B_Top_gt_pcs_rx_rst_i       ( gt_pcs_rx_rst                    ),
    .RoraLink_64B66B_Top_gt_rx_fifo_rdusewd_o  (                                  ),

    .RoraLink_64B66B_Top_tx_sys_reset_o        ( tx_sys_reset                     ), 



    .RoraLink_64B66B_Top_rx_sys_reset_o        ( rx_sys_reset                     ),


    .RoraLink_64B66B_Top_gt_pma_rstn_i         ( gt_pma_rstn                      ),
    .RoraLink_64B66B_Top_gt_pll_ok_o           ( gt_pll_ok                        ),  
    .RoraLink_64B66B_Top_gt_pcs_tx_clk_o       ( gt_pcs_tx_clk                    )

);


ll_to_axi_reg #
(
    .DATA_WIDTH                            ( DATA_WIDTH                       ),
    .STRB_WIDTH                            ( KEEP_WIDTH                       ),
    .USE_4_NFC                             ( 0                                ),
    .REM_WIDTH                             ( REM_WIDTH                        )
)                                         
u_data_ll_to_axi                          
(                                         
    // LocalLink input Interface          
    .LL_IP_DATA                            ( tx_d                             ),
                                                           
    .LL_IP_SOF_N                           ( tx_sof_n                         ),
    .LL_IP_EOF_N                           ( tx_eof_n                         ),
    .LL_IP_REM                             ( tx_rem                           ),
                                                                    
    .LL_IP_SRC_RDY_N                       ( tx_src_rdy_n                     ),
    .LL_OP_DST_RDY_N                       ( tx_dst_rdy_n                     ),

    // AXI4-S output signals              
    .AXI4_S_OP_TVALID                      ( user_tx_valid                    ),
    .AXI4_S_OP_TDATA                       ( user_tx_data                     ),
                                                         
    .AXI4_S_OP_TKEEP                       ( user_tx_strb                     ),
    .AXI4_S_OP_TLAST                       ( user_tx_last                     ),
    .AXI4_S_IP_TREADY                      ( user_tx_ready                    ),
                                   
    .USER_CLK                              ( sys_clk                          ),
    .RESET                                 ( reset_tx                         )
);

    assign ufc_tx_req = !ufc_tx_req_n;
ll_to_axi_reg #
(
    .DATA_WIDTH                            ( DATA_WIDTH                       ),
    .STRB_WIDTH                            ( 8                                ),
    .USE_4_NFC                             ( 2                                ),
    .REM_WIDTH                             ( 8                                )
)
u_ufc_ll_to_axi
(
    .LL_IP_DATA                            ( ufc_ll_tx_data                   ),
    .LL_IP_SOF_N                           (                                  ),
    .LL_IP_EOF_N                           (                                  ),
    .LL_IP_REM                             ( ufc_ll_tx_ms                     ),
    .LL_IP_SRC_RDY_N                       ( ufc_ll_tx_src_rdy_n              ),
    .LL_OP_DST_RDY_N                       ( ufc_ll_tx_dst_rdy_n              ),

    // AXI4-S output signals                                                  
    .AXI4_S_OP_TVALID                      ( ufc_tx_valid                     ),
    .AXI4_S_OP_TDATA                       ( ufc_tx_data                      ),
    .AXI4_S_OP_TKEEP                       ( ufc_tx_ms                        ),
    .AXI4_S_OP_TLAST                       (                                  ),

    .AXI4_S_IP_TREADY                      ( ufc_tx_ready                     ),

    .USER_CLK                              ( sys_clk                          ),
    .RESET                                 ( reset_tx                         )
);

generate begin
    genvar i;
    for (i = 0; i < LANE_WIDTH; i = i + 1) begin : gen_user_tx_k
        assign user_k_ll_data[LANE_WIDTH*64-64-i*64:LANE_WIDTH*64-i*64-1'b1]     =   {4'h0,user_k_blk_no[0:3],user_k_data[56*i:56*i+55]};
    end
end
endgenerate 

ll_to_axi_reg #
(
    .DATA_WIDTH                            ( DATA_WIDTH                       ),
    .STRB_WIDTH                            ( KEEP_WIDTH                       ),
    .USE_4_NFC                             ( 3                                ),
    .REM_WIDTH                             ( REM_WIDTH                        )
)
u_userk_ll_to_axi
(
    .LL_IP_DATA                            ( user_k_ll_data                   ),
    .LL_IP_SOF_N                           (                                  ),
    .LL_IP_EOF_N                           (                                  ),
    .LL_IP_REM                             (                                  ),
    .LL_IP_SRC_RDY_N                       ( user_k_src_rdy_n                 ),
    .LL_OP_DST_RDY_N                       ( user_k_dst_rdy_n                 ),

    // AXI4-S output signals                                                     
    .AXI4_S_OP_TVALID                      ( user_tx_k_valid                  ),
    .AXI4_S_OP_TDATA                       ( user_tx_k_data                   ),
    .AXI4_S_OP_TKEEP                       (                                  ),
    .AXI4_S_OP_TLAST                       (                                  ),

    .AXI4_S_IP_TREADY                      ( user_tx_k_ready                  ),

    .USER_CLK                              ( sys_clk                          ),
    .RESET                                 ( reset_tx                         )
);

// Frame Generator to provide with input to TX_Stream
frame_gen #
(
    .REM_BUS                               ( REM_WIDTH                        )
)
u_frame_gen
(
    .TX_D                                  ( tx_d                             ),                                                             
    .TX_SOF_N                              ( tx_sof_n                         ),
    .TX_EOF_N                              ( tx_eof_n                         ),
    .TX_REM                                ( tx_rem                           ),

    .TX_SRC_RDY_N                          ( tx_src_rdy_n                     ),
    .TX_DST_RDY_N                          ( tx_dst_rdy_n                     ),

    //UFC TX interface                                                        
    .UFC_TX_REQ_N                          ( ufc_tx_req_n                     ),
    .UFC_TX_MS                             ( ufc_ll_tx_ms                     ),
    .UFC_TX_DATA                           ( ufc_ll_tx_data                   ),
    .UFC_TX_SRC_RDY_N                      ( ufc_ll_tx_src_rdy_n              ),
    .UFC_TX_DST_RDY_N                      ( ufc_ll_tx_dst_rdy_n              ),

    //User K TX interface                                                    
    .USER_K_DATA                           ( user_k_data                      ),
    .USER_K_BLK_NO                         ( user_k_blk_no                    ),
    .USER_K_SRC_RDY_N                      ( user_k_src_rdy_n                 ),
    .USER_K_DST_RDY_N                      ( user_k_dst_rdy_n                 ),

    .CHANNEL_UP                            ( tx_channel_up                    ),
    .RESET4RX                              ( 1'b0                             ),

    .USER_CLK                              ( sys_clk                          ),
    .RESET                                 ( reset_tx                         )
);



axi_to_ll_reg #
(
    .DATA_WIDTH                            ( DATA_WIDTH                       ),
    .STRB_WIDTH                            ( KEEP_WIDTH                       ),
    .ISUFC                                 ( 0                                ),
    .REM_WIDTH                             ( REM_WIDTH                        )
)
u_data_axi_to_ll
(
    // AXI4-S input signals                                                     
    .AXI4_S_IP_TX_TVALID                   ( user_rx_valid                    ),
    .AXI4_S_IP_TX_TREADY                   (                                  ),
    .AXI4_S_IP_TX_TDATA                    ( user_rx_data                     ),

    .AXI4_S_IP_TX_TKEEP                    ( user_rx_strb                     ),
    .AXI4_S_IP_TX_TLAST                    ( user_rx_last                     ),
    
    // LocalLink output Interface
    .LL_OP_DATA                            ( rx_d                             ),

    .LL_OP_SOF_N                           ( rx_sof_n                         ),
    .LL_OP_EOF_N                           ( rx_eof_n                         ),
    .LL_OP_REM                             ( rx_rem                           ),

    .LL_OP_SRC_RDY_N                       ( rx_src_rdy_n                     ),
    .LL_IP_DST_RDY_N                       ( 1'b0                             ),

    .USER_CLK                              ( sys_clk                          ),
    .RESET                                 ( reset_rx                         ),
    .CHANNEL_UP                            ( rx_channel_up                    )
);

axi_to_ll_reg #
(
    .DATA_WIDTH                            ( DATA_WIDTH                       ),
    .STRB_WIDTH                            ( KEEP_WIDTH                       ),
    .ISUFC                                 ( 1                                ),
    .REM_WIDTH                             ( REM_WIDTH                        )
)
u_ufc_axi_to_ll
(
    // AXI4-S input signals                                                     
    .AXI4_S_IP_TX_TVALID                   ( ufc_rx_valid                     ),
    .AXI4_S_IP_TX_TREADY                   (                                  ),
    .AXI4_S_IP_TX_TDATA                    ( ufc_rx_data                      ),                                                          
    .AXI4_S_IP_TX_TKEEP                    ( ufc_rx_strb                      ),
    .AXI4_S_IP_TX_TLAST                    ( ufc_rx_last                      ),                                                                     
    // LocalLink output Interface                                            
    .LL_OP_DATA                            ( ufc_ll_rx_data                   ),                                                          
    .LL_OP_SOF_N                           ( ufc_ll_rx_sof_n                  ),
    .LL_OP_EOF_N                           ( ufc_ll_rx_eof_n                  ),
    .LL_OP_REM                             ( ufc_ll_rx_rem                    ),                                                                      
    .LL_OP_SRC_RDY_N                       ( ufc_ll_rx_src_rdy_n              ),
    .LL_IP_DST_RDY_N                       ( 1'b0                             ),

    .USER_CLK                              ( sys_clk                          ),
    .RESET                                 ( reset_rx                         ),
    .CHANNEL_UP                            ( rx_channel_up                    )
);

axi_to_ll_reg #
(
    .DATA_WIDTH                            ( DATA_WIDTH                       ),
    .STRB_WIDTH                            ( KEEP_WIDTH                       ),
    .ISUFC                                 ( 0                                ),
    .REM_WIDTH                             ( REM_WIDTH                        )
)
u_userk_axi_to_ll
(
    // AXI4-S input signals                                                     
    .AXI4_S_IP_TX_TVALID                   ( user_rx_k_valid                  ),
    .AXI4_S_IP_TX_TREADY                   (                                  ),
    .AXI4_S_IP_TX_TDATA                    ( user_rx_k_data                   ),                                                                     
    .AXI4_S_IP_TX_TKEEP                    ( {KEEP_WIDTH{1'b0}}               ),
    .AXI4_S_IP_TX_TLAST                    ( 1'b0                             ),

    // LocalLink output Interface                                            
    .LL_OP_DATA                            ( rx_user_k_data                   ),     
    .LL_OP_SOF_N                           (                                  ),
    .LL_OP_EOF_N                           (                                  ),
    
    .LL_OP_REM                             (                                  ),                                                                     
    .LL_OP_SRC_RDY_N                       ( ll_rx_user_k_src_rdy_n           ),
    .LL_IP_DST_RDY_N                       ( 1'b0                             ),

    .USER_CLK                              ( sys_clk                          ),
    .RESET                                 ( reset_rx                         ),
    .CHANNEL_UP                            ( rx_channel_up                    )
);

generate begin
    genvar i;
    for (i = 0; i < LANE_WIDTH; i = i + 1) begin : gen_user_rx_k
        assign ll_rx_user_k_blk_no[LANE_WIDTH*4-4-i*4:LANE_WIDTH*4-i*4-1'b1]        =   rx_user_k_data[64*i+4:64*i+7];
        assign ll_rx_user_k_data[LANE_WIDTH*56-56-i*56:LANE_WIDTH*56-i*56-1'b1]     =   rx_user_k_data[64*i+8:64*i+63];
    end
end
endgenerate 


// Frame Checker to check output from RX_Stream
frame_check #
(
    .REM_BUS                               ( REM_WIDTH                        )
)
u_frame_check
(
    .RX_D                                  ( rx_d                             ),                                                          
    .RX_REM                                ( rx_rem                           ),
    .RX_SOF_N                              ( rx_sof_n                         ),
    .RX_EOF_N                              ( rx_eof_n                         ),                                                                     
    .RX_SRC_RDY_N                          ( rx_src_rdy_n                     ),
    .DATA_ERR_COUNT                        ( data_err_count                   ),

    //User Flow Control Interface
    .UFC_RX_DATA                           ( ufc_ll_rx_data                   ),
    .UFC_RX_REM                            ( ufc_ll_rx_rem                    ),
    .UFC_RX_SOF_N                          ( ufc_ll_rx_sof_n                  ),
    .UFC_RX_EOF_N                          ( ufc_ll_rx_eof_n                  ),
    .UFC_RX_SRC_RDY_N                      ( ufc_ll_rx_src_rdy_n              ),
    .UFC_ERR_COUNT                         ( ufc_err_count                    ),
    .UFC_IN_PROGRESS                       ( ufc_in_progress                  ),
    
    // RX User K Interface
    .RX_USER_K_DATA                        ( ll_rx_user_k_data                ),
    .RX_USER_K_SRC_RDY_N                   ( ll_rx_user_k_src_rdy_n           ),
    .RX_USER_K_BLK_NO                      ( ll_rx_user_k_blk_no              ),
    .USER_K_ERR_COUNT                      ( user_k_err_count                 ),
  
    .CHANNEL_UP                            ( rx_channel_up                    ),
    .USER_CLK                              ( sys_clk                          ),
    
    .RESET4RX                              ( 1'b0                             ),

    .RESET                                 ( reset_rx                         )
);


    always @(posedge sys_clk) begin
        if(reset_rx
           ) begin
            crc_err_count <= 8'd0;
        end
        else if(crc_valid & !crc_pass_fail_n)begin
            crc_err_count <= crc_err_count + 1'b1;
        end
    end

//------------------------------------------------------------------------------//
always @(posedge sys_clk) begin
    data_err_count_1d            <=  |data_err_count;
    crc_err_count_1d             <=  |crc_err_count;
    ufc_err_count_1d             <=  |ufc_err_count;
    user_k_err_count_1d          <=  |user_k_err_count;                     
    gt_block_sync_1d             <=  &gt_rx_block_sync;
    gt_block_sync_2d             <=  gt_block_sync_1d;
end

always @(posedge sys_clk) begin
    if(|gt_rx_charerr)
        gt_err_cnt               <=  gt_err_cnt + 1'b1;
end

always @(posedge sys_clk) begin
    if(!gt_block_sync_1d & gt_block_sync_2d)
        gt_block_resync_cnt      <=  gt_block_resync_cnt + 1'b1;
end


always @(posedge sys_clk) begin
    if(reset_rx) begin
        test_pass <= 1'b0;
    end
    else if (
    (crc_err_count_1d[0] == 1'd0) &
    (ufc_err_count_1d[0] == 1'd0) &
    (user_k_err_count_1d[0] == 1'd0) &
    (data_err_count_1d[0] == 1'd0)) begin
        test_pass <= 1'b1;    
    end
    else begin
        test_pass <= 1'b0;    
    end
end

always @(posedge sys_clk) begin
    soft_err_1d                     <=  soft_err;
    hard_err_1d                     <=  hard_err;
    channel_up_1d                   <=  channel_up;
    all_lane_up_1d                  <=  &lane_up;
    channel_up_2d                   <=  channel_up_1d;
    all_lane_up_2d                  <=  all_lane_up_1d;
end
always @(posedge sys_clk) begin
    if(all_lane_up_2d & !all_lane_up_1d) begin
        lane_resync_cnt             <=  lane_resync_cnt + 1'b1;
    end
end
always @(posedge sys_clk) begin
    if(channel_up_2d & !channel_up_1d) begin
        channel_resync_cnt          <=  channel_resync_cnt + 1'b1;
    end
end
always @(posedge sys_clk) begin
    if(hard_err_1d == 1'b1) begin
        hard_err_cnt                <=  hard_err_cnt + 1'b1;
    end
end
always @(posedge sys_clk) begin
    if(soft_err_1d == 1'b1) begin
        soft_err_cnt                <=  soft_err_cnt + 1'b1;
    end
end

endmodule
